/***********************************************************************
*
* Copyright (c) 2021-2023 Barbara Geller and Ansel Sermersheim
*
* This example is free software, released under the BSD 2-Clause license.
* For license details refer to LICENSE provided with this project.
*
* CopperSpice is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
* https://opensource.org/licenses/BSD-2-Clause
*
***********************************************************************/

#include <QtCore>
#include <QtGui>

class MainWindow : public QWidget
{
 public:
   MainWindow();

 private:
   QLineEdit *textEdit;
   QComboBox *split_A;
   QComboBox *split_B;

   QLineEdit *textFormat;
   QLineEdit *arg_1;
   QLineEdit *arg_2;
   QLineEdit *result;

   void parseText();
};

MainWindow::MainWindow()
{
   setMinimumSize(700, 400);

   // part 1
   QLabel *textLabel = new QLabel("Enter Text: ");

   textEdit = new QLineEdit();
   textEdit->setText("A wacky fox and sizeable pig jumped halfway over a blue moon.");

   QLabel *label_A1 = new QLabel("split(text, ' ')");
   split_A = new QComboBox();

   QLabel *label_B1 = new QLabel("split(text, \"(fox|pig|blue)\")");
   split_B = new QComboBox();

   // part 2
   QLabel *label_A2 = new QLabel("Enter Text:");

   textFormat = new QLineEdit();
   textFormat->setText("%1 says hello in example %2");

   QLabel *label_B2 = new QLabel("Enter Arg 1:");

   arg_1 = new QLineEdit();
   arg_1->setText("CopperSpice");

   QLabel *label_C2 = new QLabel("Enter Arg 2:");

   arg_2 = new QLineEdit();
   arg_2->setText("38");

   QLabel *label_D2 = new QLabel("formatArg()");

   result = new QLineEdit();

   QFrame *hline = new QFrame();
   hline->setLineWidth(1);
   hline->setFrameStyle(QFrame::HLine);

   QPushButton *pb_run = new QPushButton();
   pb_run->setText("Run QStringParser");

   QPushButton *pb_close = new QPushButton();
   pb_close->setText("Close");

   //
   QGridLayout *grid1 = new QGridLayout();
   grid1->setHorizontalSpacing(20);
   grid1->setVerticalSpacing(15);

   grid1->addWidget(textLabel, 3, 1);
   grid1->addWidget(textEdit, 3, 2, 1, 2);

   grid1->addWidget(label_A1, 4, 2);
   grid1->addWidget(split_A, 4, 3, 1, 1);

   grid1->addWidget(label_B1, 5, 2);
   grid1->addWidget(split_B, 5, 3, 1, 1);

   grid1->addWidget(hline, 6, 1, 1, 3);

   grid1->addWidget(label_A2, 7, 1);
   grid1->addWidget(textFormat, 7, 2, 1, 1);

   grid1->addWidget(label_B2, 8, 1);
   grid1->addWidget(arg_1, 8, 2, 1, 1);

   grid1->addWidget(label_C2, 9, 1);
   grid1->addWidget(arg_2, 9, 2, 1, 1);

   grid1->addWidget(label_D2, 10, 2);
   grid1->addWidget(result, 10, 3, 1, 1);

   grid1->addItem(new QSpacerItem(100, 0,
      QSizePolicy::Fixed, QSizePolicy::Fixed), 4, 3);

   grid1->addItem(new QSpacerItem(0, 0,
      QSizePolicy::Expanding, QSizePolicy::Expanding), 13, 3);

   QHBoxLayout *layout1 = new QHBoxLayout();
   layout1->addStretch();
   layout1->addWidget(pb_run);
   layout1->addSpacing(20);
   layout1->addWidget(pb_close);
   layout1->addStretch();

   QVBoxLayout *layoutMain = new QVBoxLayout(this);
   layoutMain->setContentsMargins(20, 40, 30, 15);
   layoutMain->addLayout(grid1);
   layoutMain->addLayout(layout1);

   connect(pb_run,   &QPushButton::clicked, this, &MainWindow::parseText);
   connect(pb_close, &QPushButton::clicked, this, &QWidget::close);

   parseText();
}

void MainWindow::parseText()
{
   // part 1
   QString text = textEdit->text();

   QStringList list = QStringParser::split(text, ' ');
   split_A->clear();
   split_A->addItems(list);

   list = QStringParser::split(text,
      QRegularExpression("(fox|pig|blue)", QPatternOption::CaseInsensitiveOption));

   split_B->clear();
   split_B->addItems(list);

   // part 2
   QString str = textFormat->text();
   result->setText(str.formatArg(arg_1->text()).formatArg(arg_2->text()));
}

int main(int argc, char *argv[])
{
   QApplication app(argc, argv);

   MainWindow *window = new MainWindow();
   window->show();

   return app.exec();
}
